@@ -4,8 +4,15 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from .core import FFPuppet
-from .exceptions import BrowserTimeoutError, BrowserTerminatedError, LaunchError
+from .core import Debugger, FFPuppet, Reason
+from .exceptions import BrowserTerminatedError, BrowserTimeoutError, LaunchError
 
-__all__ = ("FFPuppet", "BrowserTimeoutError", "BrowserTerminatedError", "LaunchError")
+__all__ = (
+    "Debugger",
+    "FFPuppet",
+    "Reason",
+    "BrowserTimeoutError",
+    "BrowserTerminatedError",
+    "LaunchError",
+)
 __author__ = "Tyson Smith"
