@@ -1,16 +1,20 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
+"""browser and debugger log management"""
 
 from json import dump as json_dump
 from logging import getLogger
-from os import close as os_close, getpid, listdir, makedirs, mkdir, stat, stat_result
-from os.path import abspath, isdir, isfile, join as pathjoin, realpath
+from os import close as os_close
+from os import getpid, makedirs, mkdir, scandir, stat, stat_result
+from os.path import abspath, isdir, isfile
+from os.path import join as pathjoin
+from os.path import realpath
 from shutil import copy2, copyfileobj, copytree, rmtree
-from subprocess import CalledProcessError, check_output, STDOUT
+from subprocess import STDOUT, CalledProcessError, check_output
 from tempfile import mkdtemp, mkstemp
 
-from .helpers import onerror, wait_on_files
+from .helpers import onerror, warn_open
 
 LOG = getLogger(__name__)
 
@@ -20,7 +24,7 @@ __credits__ = ["Tyson Smith"]
 __all__ = ("PuppetLogger",)
 
 
-class PuppetLogger(object):
+class PuppetLogger:  # pylint: disable=missing-docstring
     BUF_SIZE = 0x10000  # buffer size used to copy logs
     META_FILE = "log_metadata.json"
     PATH_RR = "rr-traces"
@@ -89,14 +93,12 @@ class PuppetLogger(object):
         """
         return self._logs.keys()
 
-    def clean_up(self, ignore_errors=False, wait_delay=10):
+    def clean_up(self, ignore_errors=False):
         """Remove log files from disk.
 
         Args:
             ignore_errors (bool): Ignore errors triggered by removing files and
                                   directories will be ignored.
-            wait_delay (int): Maximum amount of time to wait for files to close
-                              if an error is hit before retrying.
 
         Returns:
             None
@@ -106,11 +108,13 @@ class PuppetLogger(object):
         if self.working_path is not None and isdir(self.working_path):
             for attempt in range(2):
                 try:
-                    rmtree(self.working_path, ignore_errors=ignore_errors, onerror=onerror)
+                    rmtree(
+                        self.working_path, ignore_errors=ignore_errors, onerror=onerror
+                    )
                 except OSError:
                     if attempt > 0:
+                        warn_open(self.working_path)
                         raise
-                    wait_on_files(self.files, timeout=wait_delay)
                     continue
                 break
         self._logs.clear()
@@ -136,9 +140,10 @@ class PuppetLogger(object):
             if offset is not None:
                 in_fp.seek(offset)
             if target_file is None:
-                cpyfp = PuppetLogger.open_unique()
+                cpyfp = PuppetLogger.open_unique(base_dir=self._base)
                 target_file = cpyfp.name
             else:
+                # pylint: disable=consider-using-with
                 cpyfp = open(target_file, "wb")
             try:
                 copyfileobj(in_fp, cpyfp, self.BUF_SIZE)
@@ -221,13 +226,10 @@ class PuppetLogger(object):
         Returns:
             file: An open file object.
         """
-        tmp_fd, log_file = mkstemp(
-            suffix=".txt",
-            prefix="ffp_log_",
-            dir=base_dir)
+        tmp_fd, log_file = mkstemp(suffix=".txt", prefix="ffp_log_", dir=base_dir)
         os_close(tmp_fd)
         # use open() so the file object 'name' attribute is correct
-        return open(log_file, mode)
+        return open(log_file, mode)  # pylint: disable=consider-using-with
 
     def reset(self):
         """Reset logger for reuse.
@@ -243,7 +245,9 @@ class PuppetLogger(object):
         self._rr_packed = False
         self.working_path = realpath(mkdtemp(prefix="ffplogs_", dir=self._base))
 
-    def save_logs(self, dest, logs_only=False, meta=False):
+    def save_logs(
+        self, dest, logs_only=False, meta=False, bin_path=None, rr_pack=False
+    ):
         """The browser logs will be saved to dest. This can only be called
         after close() has been called.
 
@@ -253,6 +257,8 @@ class PuppetLogger(object):
             logs_only (bool): Do not include other data, including debugger
                               output files.
             meta (bool): Output JSON file containing log file meta data.
+            bin_path (str): Path to Firefox binary.
+            rr_pack (bool): Pack rr trace if required.
 
         Returns:
             None
@@ -269,25 +275,35 @@ class PuppetLogger(object):
             out_name = "log_%s.txt" % log_id
             if meta:
                 file_stat = stat(log_fp.name)
-                meta_map[out_name] = {field: getattr(file_stat, field)
-                                      for field in dir(stat_result) if field.startswith("st_")}
+                meta_map[out_name] = {
+                    field: getattr(file_stat, field)
+                    for field in dir(stat_result)
+                    if field.startswith("st_")
+                }
             copy2(log_fp.name, pathjoin(dest, out_name))
 
         if not logs_only:
             rr_trace = pathjoin(self.working_path, self.PATH_RR, "latest-trace")
-            if not self._rr_packed and isdir(rr_trace):
-                LOG.debug("packing rr trace")
-                try:
-                    check_output(["rr", "pack", rr_trace], stderr=STDOUT)
-                    self._rr_packed = True
-                except (OSError, CalledProcessError):
-                    LOG.warning("Error calling 'rr pack %s'", rr_trace)
+            if isdir(rr_trace):
+                if rr_pack and not self._rr_packed:
+                    LOG.debug("packing rr trace")
+                    try:
+                        check_output(["rr", "pack", rr_trace], stderr=STDOUT)
+                        self._rr_packed = True
+                    except (OSError, CalledProcessError):
+                        LOG.warning("Error calling 'rr pack %s'", rr_trace)
+                # copy `taskcluster-build-task` for use with Pernosco if available
+                if bin_path is not None:
+                    task_info = pathjoin(bin_path, "taskcluster-build-task")
+                    if isfile(task_info):
+                        moz_rr = pathjoin(rr_trace, "files.mozilla")
+                        makedirs(moz_rr, exist_ok=True)
+                        copy2(task_info, moz_rr)
+                        LOG.debug("Copied 'taskcluster-build-task' to trace")
 
-            for path in listdir(self.working_path):
-                full_path = pathjoin(self.working_path, path)
-                if not isdir(full_path):
-                    continue
-                copytree(full_path, pathjoin(dest, path), symlinks=True)
+            for entry in scandir(self.working_path):
+                if entry.is_dir():
+                    copytree(entry.path, pathjoin(dest, entry.name), symlinks=True)
 
         if meta_map:
             with open(pathjoin(dest, self.META_FILE), "w") as json_fp:
